<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS 
 * 
 * iHRIS is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @package iHRIS
 * @subpackage Plan
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @access public
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The form class for tracking changes to a cadre pool.
 * @package iHRIS
 * @subpackage Plan
 */
class iHRIS_Change extends I2CE_Form{

    /**
     * Constant value for Intake Changes
     */
    const CHANGE_INTAKE = 1;
    /**
     * Constant value for Exit Changes
     */
    const CHANGE_EXIT = 2;
    /**
     * @var array Display names for change types.
     */
    static public $types = array( self::CHANGE_INTAKE => 'Intake', self::CHANGE_EXIT => 'Exit' );


    /**
     * Create a new instance of a projection.
     * 
     * @param string $form The name of this form
     * @param integer $id
     */
    public function __construct( $form, $id = 0 ) {                
        parent::__construct( $form, $id );
        $this->year = I2CE_Date::blank( I2CE_Date::YEAR_ONLY );
        $this->enable = true;
    }

    /**
     * Check to make sure all the fields are valid.
     * @param I2CE_Template $page
     */
    public function validate( $page ) {
        parent::validate( $page );
        $factory = I2CE_FormFactory::instance();
        $parent_obj = $factory->createForm( "pool", $this->getParent() );
        $parent_obj->populate();
        $proj = $factory->createForm( "projection", $parent_obj->getParent() );
        $proj->populate();
        $start_year = $proj->start_year->year();
        $duration = $proj->duration;
        $year = $this->year->year();
        if ( $year > 0 && ( $year < $proj->start_year->year() || $year > $start_year+$duration ) ) {
            $page->setFormInvalid( $this->getField( "year" )->getHTMLName(), "The year is outside the range for this projection." );
        }
        if ( !I2CE_Validate::checkNumber( $this->static_amount ) 
                && !I2CE_Validate::checkNumber( $this->percent_rate ) ) {
            $page->setFormInvalid( $this->getField( "static_amount" )->getHTMLName(), "You must enter either an Amount of Change or Rate of Change." );
            $page->setFormInvalid( $this->getField( "percent_rate" )->getHTMLName(), "You must enter either an Amount of Change or Rate of Change." );
        } elseif ( I2CE_Validate::checkNumber( $this->static_amount ) 
                && I2CE_Validate::checkNumber( $this->percent_rate ) ) {
            $page->setFormInvalid( $this->getField( "static_amount" )->getHTMLName(), "You must enter only an Amount of Change OR Rate of Change." );
            $page->setFormInvalid( $this->getField( "percent_rate" )->getHTMLName(), "You must enter only an Amount of Change OR Rate of Change." );
        }
    }

    /**
     * Save the object
     * 
     * Saves the object to the related database table.
     * 
     * @param User &$user The user saving this object.
     * @param boolean $transact.  Defaults to true -- all the statements of this save 
     * should be wrapped in a transaction if possible.
     */
    public function save( &$user, $transact = true) {
        if ( I2CE_Validate::checkNumber( $this->static_amount ) ) {
            $this->amount = $this->static_amount;
            if ( I2CE_Validate::checkNumber( $this->static_rate ) ) {
                $this->rate = $this->static_rate;
            }
        } elseif ( I2CE_Validate::checkNumber( $this->percent_rate ) ) {
            $this->rate = $this->percent_rate;
        }

        parent::save( $user, $transact );
    }

    /**
     * Populate the member variables of the object from the database.
     */
    public function populate() {
        parent::populate();
        if ( I2CE_Validate::checkNumber( $this->amount ) ) {
            $this->static_amount = $this->amount;
            $this->static_rate = $this->rate;
        } else {
            $this->percent_rate = $this->rate;
        }
    }


    /**
     * Lookup the given value from the type array.
     * @param integer $id
     * @param string $form Not used for this method.
     * @return string
     */
    static public function lookupType( $id, $form="" ) {
        return I2CE_Form::lookupArray( $id, self::$types );
    }

    /**
     * List all the options from the type array.
     * @param string $form The  form we wish to lookup by
     * @returns array
     */
    static public function listTypeOptions($form ) {
        return self::$types ;
    }



}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
