<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS 
 * 
 * iHRIS is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * The projection form.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The projection form.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_Projection extends I2CE_Form{

    /**
     * @var array The projected population for this projection.
     */
    private $__population;
    /**
     * @var boolean Flag to determine if the pool projection only includes the initial year's data.
     */
    private $__initial_year;

    /**
     * Create a new instance of a projection.
     * 
     * @param string $form The name of this form
     * @param integer $id
     */
    public function __construct( $form, $id = 0 ) {                
        parent::__construct( $form, $id );
        $this->start_year = I2CE_Date::now( I2CE_Date::YEAR_ONLY );
        $this->__population = array();
    }

    /**
     * Calculate the population for this projection.
     *
     * Estimate the population for all the years in this projection and return it as an array.
     * @return array
     */
    public function projectPopulation( $initial_year = false ) {
        if ( count( $this->__population ) == 0 || $this->__initial_year !== $initial_year ) {
            $this->__initial_year = $initial_year;
            $this->populate();
            $start_year = $this->start_year->year();
            $duration = $this->duration;
            $this->populateChildren( array( "population" ) );
            $real_data = array();
            foreach( $this->children['population'] as $pop ) {
                $real_data[ $pop->year->year() ] = array( 'population' => $pop->population, 'growth' => $pop->growth );
            }
            $prev_pop = 0;
            $prev_rate = 0;
            for( $year = $start_year; $duration-- >= 0; $year++ ) {
                if ( ($initial_year == false || $initial_year = $year) 
                        && array_key_exists( $year, $real_data ) ) {
                    $this->__population['pop'][$year] = $real_data[ $year ][ 'population' ];
                    $this->__population['real'][$year] = true;
                    $prev_rate = $real_data[ $year ][ 'growth' ];
                } else {
                    $this->__population['pop'][ $year ] = $prev_pop + round($prev_pop * $prev_rate / 100);
                }
                $prev_pop = $this->__population['pop'][ $year ];
            }
        }
        return $this->__population;
    }


    /**
     * Search the database for a projection record.
     * 
     * Returns a list of records that match the given search parameters.
     * The $search parameter is an array in the following format:
     * array( 
     *  '<form>' => array( 
     *          0 => array( 'field' => '<field_name>', 
     *                  'values' => array( 0 => array( 'value' => '<field_value>', 'method' => '=', 'lower' => <boolean> ) ),
     *                  - or -
     *                  'values' => 'norecord',
     *                  'value_andor' => 'OR',
     *                  'history' => <false|true>
     * )
     * 
     * $andor can be an array.  If it is, the first entry is the default value used.  The 1st entry goes between the 1st two where clauses, etc.
     * 
     * Any forms not matching projection will use the parent id from the record entry for the matching results.
     * @param array $search
     * @param string $andor If the search results should be AND or OR among all the search criteria
     * @param array $limit The limit to use on the final query, ie: array( 0, 100 ) to start at the beginning
     *                     and return 100 rows.
     * @return array
     */
    static public function search( $search, $andor = "AND", $limit=null ) {
        $db = MDB2::singleton();
        I2CE::pearError( $db, "Error getting database connection: " );

        $name_details = I2CE_FormField::getFormFieldIdAndType( "projection", "name" );
        $cat_details = I2CE_FormField::getFormFieldIdAndType( "projection", "category" );

        $params = array();
                                
        $query = "SELECT SQL_CALC_FOUND_ROWS e1.record,e1." . $name_details['type'] 
            . "_value AS name, e2." . $cat_details['type'] . "_value as category "
            . " FROM record r, last_entry e1 LEFT JOIN last_entry e2 ON e2.record = e1.record AND e2.form_field = ?"
            . " WHERE e1.record = r.id AND e1.form_field = ?";
        $params[] = $cat_details['id'];
        $params[] = $name_details['id'];
        $where = array();
        foreach( $search as $form => $form_searches ) {
            foreach(  $form_searches as $form_search ) {
                $details = I2CE_FormField::getFormFieldIdAndType( $form, $form_search['field'] );
                if ( $details == null ) continue;

                if ( array_key_exists( 'value_andor', $form_search ) ) {
                    $value_andor = $form_search['value_andor'];
                } else {
                    $value_andor = 'OR';
                }
                $add_where = "( SUBWHERE )";
                $sub_where = array();
                if ( array_key_exists('history', $form_search) && $form_search['history'] == true ) {
                    $table = "entry";
                } else {
                    $table = "last_entry";
                }
                if ( $form == "projection" ) {
                    if ( $form_search['field'] == "name" ) {
                        //$search_field = "e1." . $details['type'] . "_value";
                        // Search all entries for name instead of just the current one.
                        $search_field = $details['type'] . "_value";
                        if ( $table == "entry" ) {
                            $add_where = "e1.record IN ( SELECT record FROM entry WHERE form_field = ? AND ( SUBWHERE ) )";
                            $params[] = $details['id'];
                        } else {
                            $search_field = "e1." . $details['type'] . "_value";
                        }
                    } else {
                        $search_field = $details['type'] . "_value";
                        $add_where = "e1.record IN ( SELECT record FROM " . $table . " WHERE form_field = ? AND ( SUBWHERE ) )";
                        $params[] = $details['id'];
                    }
            } elseif ( is_array( $form_search['values'] ) ) {
                    $search_field = "e." . $details['type'] . "_value";
                    if ( array_key_exists( "parent", $form_search ) ) {
                        $add_where = "e1.record IN ( SELECT pr.parent FROM record pr WHERE pr.id IN ( SELECT r.parent FROM record r JOIN " . $table . " e ON e.record = r.id WHERE e.form_field = ? AND ( SUBWHERE ) ) )";
                    } else {
                        $add_where = "e1.record IN ( SELECT r.parent FROM record r JOIN " . $table . " e ON e.record = r.id WHERE e.form_field = ? AND ( SUBWHERE ) )";
                    }
                    $params[] = $details['id'];
                } else {
                    $where[] = "e1.record NOT IN ( SELECT parent FROM record WHERE form = ? )";
                    $params[] = I2CE_Form::getFormId( $form );
                }
                if ( is_array( $form_search['values'] ) ) {
                    foreach( $form_search['values'] as $value ) {
                        if ( array_key_exists( 'method', $value ) ) {
                            $method = $value['method'];
                        } else {
                            $method = "=";
                        }
                        if ( $method == "ISNULL" ) {
                            $sub_where[] = $search_field . " IS NULL";
                            continue;
                        }
                        if ( array_key_exists( 'lower', $value ) ) {
                            $lower = $value['lower'];
                        } else {
                            $lower = false;
                        }
                        if ( $value['value'] == '%%' ) continue;
                        $sub_where[] = ($lower ? "LOWER(" : "" ) . $search_field . ($lower ? ")" : "") . " " . $method . " ?";
                        $params[] = $value['value'];
                    }
                    if ( count( $sub_where ) > 0 ) {
                        $where[] = str_replace( "SUBWHERE", implode( " " . $value_andor . " ", $sub_where ), $add_where );
                    }
                }
            }
        }
        
        if ( count( $where ) > 0 ) {
            if ( is_array( $andor ) && count($andor) > 1 ) {
                $where_clause = "";
                for( $i = 1; $i < count( $andor ); $i++ ) {
                    if ( $where_clause == "" ) {
                        $where_clause = "( " . $where[$i-1] . " " . $andor[$i] . " " . $where[$i] . " )";
                    } else {
                        $where_clause = "( " . $where_clause . " " . $andor[$i] . " " . $where[$i] . " )";
                    }
                }
                if ( count($where) > count($andor) ) {
                    $where_clause = $where_clause . " " . $andor[0] . " (" . implode( " " . $andor[0]. " ", array_slice( $where, count($andor) ) ) . " )";
                }
                $query .= " AND ( " . $where_clause . " )"; 
            } else {
                if ( is_array( $andor ) ) $andor = $andor[0];
                $query .= " AND ( " . implode( " " . $andor . " ", $where ) . " )";
            }
        }
        $query .= " ORDER BY name";
        if ( is_array( $limit ) && count($limit) > 0 && count($limit) < 3 ) {
            $query .= " LIMIT " . implode( ',', $limit );
        }
        $search_results = array();
        $sth = $db->prepare( $query, null, MDB2_PREPARE_RESULT );
        if ( !I2CE::pearError( $sth, "Error preparing projection search: " ) ) {
            $results = $sth->execute( $params );
            if ( !I2CE::pearError( $results, "Error executing projection search: " ) ) {
                if ( is_array( $limit ) ) {
                    $num_rows = $db->queryRow( "SELECT FOUND_ROWS() AS num_rows" );
                    if ( !I2CE::pearError( $num_rows, "Error counting projection search results: " ) ) {
                        $search_results['count'] = $num_rows->num_rows;
                    } else {
                        $search_results['count'] = "?";
                    }
                }
                while ( $data = $results->fetchRow() ) {
                    $search_results[ $data->record ] = array( "name" => $data->name, 
                            "category" => $data->category );
                }
                $results->free();
            }
            $sth->free();
        }
        return $search_results;
    }


}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
